#include "Arduino.h"
#include "Button.h"

Button::Button(uint8_t port, bool ENABLE_PULLUP) {
    if (ENABLE_PULLUP) pinMode(port, INPUT_PULLUP);
    else pinMode(port, INPUT);

    _port = port;
}


void Button::init() {
    // Nastaveni casovace na 1 ms (1/16MHz * 128 prescaler * (255-130=125) pocatecni krok)
    noInterrupts();

    TCCR2A = 0;
    TCNT2 = 130;    // Nastavime citac na 130 z 255
    TIFR2 = 0;      // Vynulovani priznaku preteceni
    //OCR2A = 4;      
    TCCR2B = 5;     // Nastaveni nasobice na 128
    TIMSK2 = 1;     // Nastaveni preruseni na preteceni

    interrupts();
}

bool Button::getState() {
    return _state;
}

void Button::updateState() {

    static uint16_t state = 0;                              // Nejspolehlivejsi osetreni zakmitu
                                                            // Nechame 1 kolovat v state. Maskou 0xa000 (1010000000000000) dame do vyssich bitu log.1. krom predposledniho bitu
                                                            // Pokud prijde impulz z tlacitka, prijdou na chvili nahodne 0 a 1.
                                                            // Az skonci zakmit, prijdou stabilni log. 1 ktere se dostanou do vyssich bitu
                                                            // Porovnanim s 0xe000 (1110000000000000) zjistime pritomnost log.1 az se dostane na predposledni bit log 1
    state = (state << 1) | digitalRead(_port) | 0xa000;
    if(state == 0xe000) _state = 1;
    else _state = 0;

    TCNT2 = 130;    // Nastavime citac na 130 z 255
}


uint8_t Button::holdTime() {
    uint8_t ret = 0;

    if (_state != _lastState) {

        // Zmacknute
        if (_state) {
            casStisku = millis();
            casRozepnuti = casStisku - konecStisku;

        // Rozeple
        } else {
            konecStisku = millis();

            uint16_t time = (konecStisku - casStisku);

            if (time <= 1500) {
                Serial.println("Stisknuto kratce");
                ret = 1;
            }
            else if (time >= 1500) {
                Serial.println("Stisknuto dlouze");
                ret = 2;
            }
        }
    }

    _lastState = _state;
    return ret;
}